"""

:Author: Franziska Koehn
:Created: 2015/01/13

This module houses classes relating to the resultstree.

"""

import gtk

class ResultsDownloadView(gtk.VBox):

    def toggle_selection(self, *_):
        self.treeViewResults.toggle_selection()

    def get_selected_items(self):
        results=[]
        for r in self.get_store():
            if r[0]:
                results.append(r)
        return results

    def get_store(self):
        return self.treeViewResults.store

    def show_data(self, results, root_type):
        self.treeViewResults.show_data(results, root_type)

    def __init__(self, *args, **kwargs):
        super(ResultsDownloadView, self).__init__(*args, **kwargs)

        sw_results = gtk.ScrolledWindow()
        sw_results.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw_results.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.pack_start(sw_results,expand=True, fill=True)

        self.treeViewResults = TreeViewResults()
        sw_results.add(self.treeViewResults)


class TreeViewResults(gtk.TreeView):

    store = []
    def __init__(self):
        super(TreeViewResults, self).__init__()
        self.show_data([], None)
        import gobject
        gobject.timeout_add(150, self.move_spinners)

    def move_spinners(self):
        for r in self.store:
            if r[2]: # if spinner is visible
                r[3] += 1
        return True


    def create_columns(self, column_labels):

        store = []

        for c in self.get_columns():
            self.remove_column(c)

        def callback_toggled(cellrenderertoggle, path_string, col, *_):
            it = self.store.get_iter_from_string(path_string)
            is_active = cellrenderertoggle.get_active()
            self.store.set(it, col, not is_active)

        renderer = gtk.CellRendererToggle()
        renderer.set_property('activatable', True)
        renderer.connect("toggled", callback_toggled, 0)
        column = gtk.TreeViewColumn("", renderer)
        column.add_attribute(renderer, "active", 0)
        column.set_sort_column_id(0)
        column.set_resizable(False)
        self.append_column(column)

        renderer = gtk.CellRendererSpinner()
        column = gtk.TreeViewColumn("", renderer, active=2, pulse=3)
        self.append_column(column)

        for i,cl in enumerate(column_labels, start=4):
            column = gtk.TreeViewColumn(cl, gtk.CellRendererText(), text=i)
            column.set_sort_column_id(i)
            column.set_resizable(True)
            self.append_column(column)

    def toggle_selection(self):
        for row in self.store:
            row[0] = not row[0]

    def show_data(self, data, root_type):
        import gobject

        if data == []:
            if self.store != []:
                self.store.clear() #TODO
            self.create_columns([])
            return

        keys = data.headers()

        model_columns = []
        model_columns.append(bool)                  # 0 is selected (for download selection)
        model_columns.append(gobject.TYPE_PYOBJECT) # 1 query result
        model_columns.append(bool)                  # 2 is spinner visible
        model_columns.append(int)                   # 3 spinner pulse
        for k in keys:
            model_columns.append(str)

        self.store = gtk.ListStore(*model_columns)

        for d in data:
            result = []
            result.append(True)
            result.append(d)
            result.append(False)
            result.append(0)
            for k in keys:
                result.append(d[k])

            self.store.append(result)

        import xsa.datatypereader as type_reader
        keys_ = list(type_reader.get_field_label_by_key(root_type, key) for key in keys)
        self.create_columns(keys_)
        self.set_model(self.store)
